'' =================================================================================================
''
''   File....... jm_adafruit_ble_connect.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2017 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 17 OCT 2017
''
'' =================================================================================================

{

  Note: CTS must be connected to ground
        Put module into CMD mode to experiment with AT commands
        Put module into UART mode to experiment with Bluefuit LE app

}


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  TRM_BAUD = 115_200                                            ' terminal baud rate
  

con { io pins }                                                  
                                                                 
  RX1  = 31  { I }                                              ' serial / programming
  TX1  = 30  { O }                                            
                                                              
  SDA1 = 29  { I/O }                                            ' i2c / eeprom
  SCL1 = 28  { I/O } 

  RX2  =  1  { I }                                              ' bluetooth
  TX2  =  0  { O }


con

  C_SIZE =  32                                                  ' command buffer size
  R_SIZE = 256                                                  ' response buffer size
  

obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  tbuf : "jm_textbuf"                                           '   text buffer management 
  btle : "jm_fullduplexserial"                                  ' * serial IO for BLE module    
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

  byte  buffer[C_SIZE]                                          ' command buffer --> btle
  
  byte  response[R_SIZE]                                        ' response <-- btle
  byte  rlen                                                    ' bytes in repsonse
                                                                 
                                                                 
pub main                                                      
                                                                 
  setup

  time.pause(50)
  term.rxflush
  btle.rxflush                                                        
                                                                 
  repeat
    get_command
    show_response    


pub get_command : b                                             ' device must be in CMD mode

  repeat
    b := term.rxcheck                                           ' anything in kbd buffer
    if (b < 0)                                                  ' if no
      quit                                                      '  bail out of loop
    else
      if (b == 8)                                               ' backspace?
        tbuf.backspace                                          '  remove last character from buffer
      else
        tbuf.putc(tbuf.upper(b))                                ' else put new char in  
      if (b == 13)                                              ' done with command
        btle.str(tbuf.str)                                      '  yes, send to BTLE
        term.str(string("--> "))                                '  display command in terminal
        term.str(tbuf.str)
        btle.txflush                                            '  let command finish
        

pub show_response | b, idx

  b := btle.rxcheck                                             ' anything in buffer
  if (b < 0)                                                    ' no
    return                                                      '  we're outta here

  bytefill(@response, 0, R_SIZE+1)                              ' clear response + len
  
  repeat
    response[rlen++] := b                                       ' put byte into response buffer
    b := btle.rxtime(3)                                         ' get the next 
    if (b < 0)                                                  ' all done? (serial timeout)
      quit

      
  ' show length of response

  term.str(string("<-- "))
  term.dec(rlen)
  term.str(string(" bytes", 13))

      
  ' show as ASCII
  ' -- particularly for use in command mode

  term.str(string("<-- "))
  repeat idx from 0 to rlen-1
    b := response[idx] 
    case b
      10      : { ignore }
      13      : term.str(string(13, "    "))                    ' CR + pad 
      32..127 : term.tx(b)
      other   : term.tx(".")     

  term.tx(13) 

  
  ' show as raw bytes (hex format)
  ' -- for use in UART mod with Bluefruit LE app

  if (tbuf.size == 0)                                           ' if AT command string
    term.str(string("<-- "))                                    
    repeat idx from 0 to rlen-1                                 ' show incoming btyes 
      term.hex(response[idx], 2)                                '  as hex
      term.tx(" ")
    term.tx(13)
 
  term.tx(13)   

  tbuf.clear                                                    ' delete command buffer
      
                                                                                
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  tbuf.use(@buffer, C_SIZE-1)                                   ' setup command buffer editing
  tbuf.clear

  btle.start(RX2, TX2, %0000, 9600)                             ' start serial for BTLE  module
                                                              
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal

                          
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      